"use strict";

exports.__esModule = true;
exports.useIds = useIds;
exports.useId = void 0;

var _vue = require("vue");

/**
 * Credit: https://github.com/reach/reach-ui/blob/develop/packages/auto-id/src/index.tsx
 *
 * Why does this hook exist?
 *   1. Accessibiliy APIs rely heavily on element IDs
 *   2. Requiring developers to put IDs on every Chakra component
 *      is cumbersome and error-prone.
 *   3. With a components model, we can generate IDs for them!
 *
 * Solutions to ID problem:
 * 1. Generate random IDs
 *    In v0.x of @chakra-ui/vue, we did this for components.
 *    Since then, we've learned some things about performance for
 *    components especially with SSR.
 *
 *    This may not be a good idea because during server rendering
 *    the IDs will be statically generated, and during client-side hydration,
 *    the IDs may not match, when booting up the Vue App. Vue will then
 *    go ahead and recreate the entire application.
 *
 * 2. Don't server render IDs. Instead patch on client `onMounted`
 *    In this approach, generated ID is an empty string on the first render.
 *    This way the client and server possess the same ID.
 *
 *    When the component is finally mounted, we patch the ID.
 *    This may cause a re-render on the client, but it shouldn't be a
 *    big problem, because:
 *
 *        1. Components using `useId` composable are small
 *        2. With solution 1, it would cause a re-render anyway.
 *        3. This patch only runs once. (Only when the `onMounted` life
 *           -cycle hook is called.)
 *
 */
var serverHandoffComplete = false;
var _id = 0;

var genId = function genId() {
  return ++_id;
};
/**
 * Generates a unique id
 *
 * @param id external ID provided by consumer/user.
 * @param prefix prefix to append before the id
 */


var useId = function useId(id, prefix) {
  var initialId = id || (serverHandoffComplete ? genId() : null);
  var uid = (0, _vue.ref)(initialId);
  (0, _vue.onBeforeMount)(function () {
    if (serverHandoffComplete === false) {
      serverHandoffComplete = true;
    }
  });
  (0, _vue.onMounted)(function () {
    if (uid.value === null) {
      uid.value = genId();
    }
  });
  return (0, _vue.computed)(function () {
    var __id__ = uid.value !== null ? uid.value.toString() : undefined;

    return prefix ? prefix + "-" + __id__ : __id__;
  });
};
/**
 * Hook to generate ids for use in compound components
 *
 * @param id the external id passed from the user
 * @param prefixes array of prefixes to use
 */


exports.useId = useId;

function useIds(id) {
  var __id__ = useId(id);

  for (var _len = arguments.length, prefixes = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    prefixes[_key - 1] = arguments[_key];
  }

  return prefixes.map(function (prefix) {
    return (0, _vue.computed)(function () {
      return prefix + "-" + __id__.value;
    });
  });
}
//# sourceMappingURL=use-id.js.map